{% load static %}
<html>
    <head>
        <title>Ga naar DigID</title>
    </head>
    <body>
        <script src="{% static 'login_submit.js' %}"></script>
        <form method="post" action="{{ url }}" name="login">
            {{ form.as_p }}
            <input type="submit" value="Ga naar DigID" />
        </form>
    </body>
</html>